@@ -328,18 +328,25 @@ sub _encode_value {
     return _encode_object($e, !!$$value, $BOOL) if $ref eq 'SCALAR';
   }
 
-  # Double
+  # Numeric
   my $flags = B::svref_2object(\$value)->FLAGS;
-  return DOUBLE . $e . pack('d<', $value) if $flags & B::SVp_NOK;
+  if ($flags & (B::SVp_IOK | B::SVp_NOK)) {
+    no warnings 'numeric';
+    my $num = $value;
+    $num += 0;
 
-  if ($flags & B::SVp_IOK) {
+    if ($num eq $value && $num * 0 == 0) {
 
-    # Int32
-    return INT32 . $e . pack('l<', $value)
-      if $value <= INT32_MAX && $value >= INT32_MIN;
+      # Double
+      return DOUBLE . $e . pack('d<', $num) if $flags & B::SVp_NOK;
 
-    # Int64
-    return INT64 . $e . pack('q<', $value);
+      # Int32
+      return INT32 . $e . pack('l<', $num)
+        if $num <= INT32_MAX && $num >= INT32_MIN;
+
+      # Int64
+      return INT64 . $e . pack('q<', $num);
+    }
   }
 
   # String
